@@ -2,8 +2,11 @@
 
 BUILDBOT_DIR=$WORKSPACE/nightly_build
 THEANO_PARAM="theano --with-timer --timer-top-n 10"
+
+export MKL_THREADING_LAYER=GNU
 # Set test reports using nosetests xunit
 XUNIT="--with-xunit --xunit-file="
+SUITE="--xunit-testsuite-name="
 
 mkdir -p ${BUILDBOT_DIR}
 ls -l ${BUILDBOT_DIR}
@@ -13,4 +16,5 @@ echo
 
 FILE=${BUILDBOT_DIR}/theano_python32bit_tests.xml
 set -x
-PYTHONPATH=$HOME/anaconda2_32bit/lib/python2.7/site-packages THEANO_FLAGS=device=cpu,force_device=true,lib.amdlibm=False,compiledir=$WORKSPACE/compile/theano_compile_dir_theano_python2_32bit $HOME/anaconda2_32bit/bin/python bin/theano-nose ${THEANO_PARAM} ${XUNIT}${FILE}
+NAME=python2_32bit
+THEANO_FLAGS=device=cpu,force_device=true,lib.amdlibm=False,compiledir=$HOME/.theano/buildbot_theano_python2_32bit python bin/theano-nose ${THEANO_PARAM} ${XUNIT}${FILE}  ${SUITE}${NAME}
