@@ -1,5 +1,5 @@
 from __future__ import absolute_import, print_function, division
-import numpy
+import numpy as np
 import unittest
 
 import theano
@@ -8,6 +8,7 @@ import theano.tensor as T
 from theano.tensor.nnet.neighbours import images2neibs, neibs2images, Images2Neibs
 
 from theano.tests import unittest_tools
+from theano import change_flags
 
 mode_without_gpu = theano.compile.mode.get_default_mode().excluding('gpu')
 
@@ -25,7 +26,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
             for border in ['valid', 'ignore_borders']:
                 for dtype in self.dtypes:
                     images = shared(
-                        numpy.arange(numpy.prod(shape), dtype=dtype).reshape(shape))
+                        np.arange(np.prod(shape), dtype=dtype).reshape(shape))
                     neib_shape = T.as_tensor_variable(pshape)
 
                     f = function([],
@@ -42,13 +43,13 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                                 for node in f.maker.fgraph.toposort()])
 
                     # print g()
-                    assert numpy.allclose(images.get_value(borrow=True), g())
+                    assert np.allclose(images.get_value(borrow=True), g())
 
     def test_neibs_manual(self):
         shape = (2, 3, 4, 4)
         for dtype in self.dtypes:
             images = shared(
-                numpy.arange(numpy.prod(shape), dtype=dtype).reshape(shape))
+                np.arange(np.prod(shape), dtype=dtype).reshape(shape))
             neib_shape = T.as_tensor_variable((2, 2))
 
             for border in ['valid', 'ignore_borders']:
@@ -60,7 +61,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                 # print images.get_value(borrow=True)
                 neibs = f()
                 # print neibs
-                assert numpy.allclose(neibs, [
+                assert np.allclose(neibs, [
                     [0, 1, 4, 5],
                     [2, 3, 6, 7],
                     [8, 9, 12, 13],
@@ -88,13 +89,13 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                 g = function([], neibs2images(neibs, neib_shape, images.shape),
                              mode=self.mode)
 
-                assert numpy.allclose(images.get_value(borrow=True), g())
+                assert np.allclose(images.get_value(borrow=True), g())
 
     def test_neibs_manual_step(self):
         shape = (2, 3, 5, 5)
         for dtype in self.dtypes:
-            images = shared(numpy.asarray(numpy.arange(numpy.prod(
-                            shape)).reshape(shape), dtype=dtype))
+            images = shared(np.asarray(np.arange(np.prod(
+                shape)).reshape(shape), dtype=dtype))
             neib_shape = T.as_tensor_variable((3, 3))
             neib_step = T.as_tensor_variable((2, 2))
             for border in ['valid', 'ignore_borders']:
@@ -107,7 +108,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                 assert self.op in [type(node.op)
                                    for node in f.maker.fgraph.toposort()]
 
-                assert numpy.allclose(neibs, [
+                assert np.allclose(neibs, [
                     [0, 1, 2, 5, 6, 7, 10, 11, 12],
                     [2, 3, 4, 7, 8, 9, 12, 13, 14],
                     [10, 11, 12, 15, 16, 17, 20, 21, 22],
@@ -140,11 +141,12 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                 # print g()
                 # assert numpy.allclose(images.get_value(borrow=True), g())
 
+    @change_flags(compute_test_value='off')
     def test_neibs_bad_shape(self):
         shape = (2, 3, 10, 10)
         for dtype in self.dtypes:
-            images = shared(numpy.arange(
-                numpy.prod(shape), dtype=dtype).reshape(shape))
+            images = shared(np.arange(
+                np.prod(shape), dtype=dtype).reshape(shape))
 
             for neib_shape in [(3, 2), (2, 3)]:
                 neib_shape = T.as_tensor_variable(neib_shape)
@@ -212,11 +214,11 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
 
             for dtype in self.dtypes:
 
-                images = shared(numpy.asarray(numpy.arange(numpy.prod(
-                                shape)).reshape(shape), dtype=dtype))
+                images = shared(np.asarray(np.arange(np.prod(
+                    shape)).reshape(shape), dtype=dtype))
                 neib_shape = T.as_tensor_variable(neib_shape)
                 neib_step = T.as_tensor_variable(neib_step)
-                expected = numpy.asarray(expected)
+                expected = np.asarray(expected)
 
                 f = function([], images2neibs(images, neib_shape, neib_step,
                                               mode="wrap_centered"),
@@ -225,7 +227,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
 
                 if expected.size > 1:
                     for i in range(shape[0] * shape[1]):
-                        assert numpy.allclose(
+                        assert np.allclose(
                             neibs[i * expected.shape[0]:(i + 1) * expected.shape[0], :],
                             expected + 25 * i), "wrap_centered"
 
@@ -236,13 +238,65 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                 # TODO: why this is commented?
                 # assert numpy.allclose(images.get_value(borrow=True), g())
 
+    def test_neibs_half_step_by_valid(self):
+        neib_shapes = ((3, 3), (3, 5), (5, 3))
+        for shp_idx, (shape, neib_step) in enumerate([
+            [(7, 8, 5, 5), (1, 1)],
+            [(7, 8, 5, 5), (2, 2)],
+            [(7, 8, 5, 5), (4, 4)],
+            [(7, 8, 5, 5), (1, 4)],
+            [(7, 8, 5, 5), (4, 1)],
+            [(80, 90, 5, 5), (1, 2)],
+            [(1025, 9, 5, 5), (2, 1)],
+            [(1, 1, 5, 1037), (2, 4)],
+            [(1, 1, 1045, 5), (4, 2)]]
+        ):
+            for neib_shape in neib_shapes:
+                for dtype in self.dtypes:
+                    x = theano.shared(np.random.randn(*shape).astype(dtype))
+                    extra = (neib_shape[0] // 2, neib_shape[1] // 2)
+                    padded_shape = (x.shape[0], x.shape[1], x.shape[2] + 2 * extra[0], x.shape[3] + 2 * extra[1])
+                    padded_x = T.zeros(padded_shape)
+                    padded_x = T.set_subtensor(padded_x[:, :, extra[0]:-extra[0], extra[1]:-extra[1]], x)
+                    x_using_valid = images2neibs(padded_x, neib_shape, neib_step, mode="valid")
+                    x_using_half = images2neibs(x, neib_shape, neib_step, mode="half")
+                    f_valid = theano.function([], x_using_valid, mode='FAST_RUN')
+                    f_half = theano.function([], x_using_half, mode=self.mode)
+                    unittest_tools.assert_allclose(f_valid(), f_half())
+
+    def test_neibs_full_step_by_valid(self):
+        for shp_idx, (shape, neib_step, neib_shapes) in enumerate([
+            [(7, 8, 5, 5), (1, 1), ((3, 3), (3, 5), (5, 3))],
+            [(7, 8, 5, 5), (2, 2), ((3, 3), (3, 5), (5, 3))],
+            [(7, 8, 6, 6), (3, 3), ((2, 2), (2, 5), (5, 2))],
+            [(7, 8, 6, 6), (1, 3), ((2, 2), (2, 5), (5, 2))],
+            [(7, 8, 6, 6), (3, 1), ((2, 2), (2, 5), (5, 2))],
+            [(80, 90, 5, 5), (1, 2), ((3, 3), (3, 5), (5, 3))],
+            [(1025, 9, 5, 5), (2, 1), ((3, 3), (3, 5), (5, 3))],
+            [(1, 1, 11, 1037), (2, 3), ((3, 3), (5, 3))],
+            [(1, 1, 1043, 11), (3, 2), ((3, 3), (3, 5))]]
+        ):
+            for neib_shape in neib_shapes:
+                for dtype in self.dtypes:
+                    x = theano.shared(np.random.randn(*shape).astype(dtype))
+                    extra = (neib_shape[0] - 1, neib_shape[1] - 1)
+                    padded_shape = (x.shape[0], x.shape[1], x.shape[2] + 2 * extra[0], x.shape[3] + 2 * extra[1])
+                    padded_x = T.zeros(padded_shape)
+                    padded_x = T.set_subtensor(padded_x[:, :, extra[0]:-extra[0], extra[1]:-extra[1]], x)
+                    x_using_valid = images2neibs(padded_x, neib_shape, neib_step, mode="valid")
+                    x_using_full = images2neibs(x, neib_shape, neib_step, mode="full")
+                    f_valid = theano.function([], x_using_valid, mode='FAST_RUN')
+                    f_full = theano.function([], x_using_full, mode=self.mode)
+                    unittest_tools.assert_allclose(f_valid(), f_full())
+
+    @change_flags(compute_test_value='off')
     def test_neibs_bad_shape_wrap_centered(self):
         shape = (2, 3, 10, 10)
 
         for dtype in self.dtypes:
-            images = shared(numpy.arange(
-                numpy.prod(shape), dtype=dtype
-                ).reshape(shape))
+            images = shared(np.arange(
+                np.prod(shape), dtype=dtype
+            ).reshape(shape))
 
             for neib_shape in [(3, 2), (2, 3)]:
                 neib_shape = T.as_tensor_variable(neib_shape)
@@ -253,7 +307,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
                 self.assertRaises(TypeError, f)
 
             for shape in [(2, 3, 2, 3), (2, 3, 3, 2)]:
-                images = shared(numpy.arange(numpy.prod(shape)).reshape(shape))
+                images = shared(np.arange(np.prod(shape)).reshape(shape))
                 neib_shape = T.as_tensor_variable((3, 3))
                 f = function([], images2neibs(images, neib_shape,
                                               mode="wrap_centered"),
@@ -262,7 +316,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
 
             # Test a valid shapes
             shape = (2, 3, 3, 3)
-            images = shared(numpy.arange(numpy.prod(shape)).reshape(shape))
+            images = shared(np.arange(np.prod(shape)).reshape(shape))
             neib_shape = T.as_tensor_variable((3, 3))
 
             f = function([],
@@ -273,7 +327,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
     def test_grad_wrap_centered(self):
         # It is not implemented for now. So test that we raise an error.
         shape = (2, 3, 6, 6)
-        images_val = numpy.random.rand(*shape).astype('float32')
+        images_val = np.random.rand(*shape).astype('float32')
 
         def fn(images):
             return images2neibs(images, (3, 3), mode='wrap_centered')
@@ -281,9 +335,31 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
         self.assertRaises(TypeError, unittest_tools.verify_grad,
                           fn, [images_val], mode=self.mode)
 
+    def test_grad_half(self):
+        # It is not implemented for now. So test that we raise an error.
+        shape = (2, 3, 6, 6)
+        images_val = np.random.rand(*shape).astype('float32')
+
+        def fn(images):
+            return images2neibs(images, (3, 3), mode='half')
+
+        self.assertRaises(TypeError, unittest_tools.verify_grad,
+                          fn, [images_val], mode=self.mode)
+
+    def test_grad_full(self):
+        # It is not implemented for now. So test that we raise an error.
+        shape = (2, 3, 6, 6)
+        images_val = np.random.rand(*shape).astype('float32')
+
+        def fn(images):
+            return images2neibs(images, (3, 3), mode='full')
+
+        self.assertRaises(TypeError, unittest_tools.verify_grad,
+                          fn, [images_val], mode=self.mode)
+
     def test_grad_valid(self):
         shape = (2, 3, 6, 6)
-        images_val = numpy.random.rand(*shape).astype('float32')
+        images_val = np.random.rand(*shape).astype('float32')
 
         def fn(images):
             return images2neibs(images, (2, 2))
@@ -305,7 +381,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
 
     def test_grad_ignore_border(self):
         shape = (2, 3, 5, 5)
-        images_val = numpy.random.rand(*shape).astype('float32')
+        images_val = np.random.rand(*shape).astype('float32')
 
         def fn(images):
             return images2neibs(images, (2, 2),
@@ -317,7 +393,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
     def test_neibs2images_grad(self):
         # say we had images of size (2, 3, 10, 10)
         # then we extracted 2x2 neighbors on this, we get (2 * 3 * 5 * 5, 4)
-        neibs_val = numpy.random.rand(150, 4)
+        neibs_val = np.random.rand(150, 4)
 
         def fn(neibs):
             return neibs2images(neibs, (2, 2), (2, 3, 10, 10))
@@ -327,18 +403,36 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
     def test_neibs_valid_with_inconsistent_borders(self):
         shape = (2, 3, 5, 5)
         images = T.dtensor4()
-        images_val = numpy.arange(numpy.prod(shape),
-                                  dtype='float32').reshape(shape)
-
-        def fn(images):
-            return T.sum(T.sqr(images2neibs(images, (2, 2), mode='valid')),
-                         axis=[0, 1])
+        images_val = np.arange(np.prod(shape),
+                               dtype='float32').reshape(shape)
 
         f = theano.function([images],
                             T.sqr(images2neibs(images, (2, 2), mode='valid')),
                             mode=self.mode)
         self.assertRaises(TypeError, f, images_val)
 
+    def test_neibs_half_with_inconsistent_borders(self):
+        shape = (2, 3, 5, 5)
+        images = T.dtensor4()
+        images_val = np.arange(np.prod(shape),
+                               dtype='float32').reshape(shape)
+
+        f = theano.function([images],
+                            T.sqr(images2neibs(images, (2, 2), mode='half')),
+                            mode=self.mode)
+        self.assertRaises(TypeError, f, images_val)
+
+    def test_neibs_full_with_inconsistent_borders(self):
+        shape = (2, 3, 5, 5)
+        images = T.dtensor4()
+        images_val = np.arange(np.prod(shape),
+                               dtype='float32').reshape(shape)
+
+        f = theano.function([images],
+                            T.sqr(images2neibs(images, (2, 2), mode='full')),
+                            mode=self.mode)
+        self.assertRaises(TypeError, f, images_val)
+
     def test_can_not_infer_nb_dim(self):
         # Was reported in gh-5613. Test that we do not crash
         # or that we crash in a few other case found while
@@ -346,7 +440,7 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
 
         img = T.tensor4('img')
         patches = T.nnet.neighbours.images2neibs(img, [16, 16])
-        extractPatches = theano.function([img], patches)
+        extractPatches = theano.function([img], patches, mode=self.mode)
 
         patsRecovery = T.matrix('patsRecovery')
         original_size = T.ivector('original_size')
@@ -354,22 +448,24 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
         for mode in ['valid', 'ignore_borders']:
             out = neibs2images(patsRecovery, (16, 16),
                                original_size, mode=mode)
-            f = theano.function([patsRecovery, original_size], out)
+            f = theano.function([patsRecovery, original_size], out,
+                                mode=self.mode)
 
-            im_val = numpy.ones((1, 3, 320, 320), dtype=numpy.float32)
+            im_val = np.ones((1, 3, 320, 320), dtype=np.float32)
             neibs = extractPatches(im_val)
             f(neibs, im_val.shape)
             # Wrong number of dimensions
             self.assertRaises(ValueError, f, neibs,
                               (1, 1, 3, 320, 320))
             # End up with a step of 0
-            self.assertRaises(ValueError, f, neibs,
+            # This can lead to division by zero in DebugMode
+            self.assertRaises((ValueError, ZeroDivisionError), f, neibs,
                               (3, 320, 320, 1))
 
     def speed_neibs(self):
         shape = (100, 40, 18, 18)
-        images = shared(numpy.arange(numpy.prod(shape),
-                                     dtype='float32').reshape(shape))
+        images = shared(np.arange(np.prod(shape),
+                                  dtype='float32').reshape(shape))
         neib_shape = T.as_tensor_variable((3, 3))
 
         f = function([], images2neibs(images, neib_shape),
@@ -380,8 +476,8 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
 
     def speed_neibs_wrap_centered(self):
         shape = (100, 40, 18, 18)
-        images = shared(numpy.arange(numpy.prod(shape),
-                                     dtype='float32').reshape(shape))
+        images = shared(np.arange(np.prod(shape),
+                                  dtype='float32').reshape(shape))
         neib_shape = T.as_tensor_variable((3, 3))
 
         f = function([],
@@ -391,9 +487,35 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
         for i in range(1000):
             f()
 
+    def speed_neibs_half(self):
+        shape = (100, 40, 18, 18)
+        images = shared(np.arange(np.prod(shape),
+                                  dtype='float32').reshape(shape))
+        neib_shape = T.as_tensor_variable((3, 3))
+
+        f = function([],
+                     images2neibs(images, neib_shape, mode="half"),
+                     mode=self.mode)
+
+        for i in range(1000):
+            f()
+
+    def speed_neibs_full(self):
+        shape = (100, 40, 18, 18)
+        images = shared(np.arange(np.prod(shape),
+                                  dtype='float32').reshape(shape))
+        neib_shape = T.as_tensor_variable((3, 3))
+
+        f = function([],
+                     images2neibs(images, neib_shape, mode="full"),
+                     mode=self.mode)
+
+        for i in range(1000):
+            f()
+
     def test_infer_shape(self):
         shape = (100, 40, 6, 3)
-        images = numpy.ones(shape).astype('float32')
+        images = np.ones(shape).astype('float32')
         x = T.ftensor4()
         self._compile_and_check(
             [x], [images2neibs(x, neib_shape=(2, 1), mode='valid')],
@@ -402,14 +524,14 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
             [x], [images2neibs(x, neib_shape=(2, 3), mode='valid')],
             [images], Images2Neibs)
         shape = (100, 40, 5, 4)
-        images = numpy.ones(shape).astype('float32')
+        images = np.ones(shape).astype('float32')
         x = T.ftensor4()
         self._compile_and_check(
             [x], [images2neibs(
                 x, neib_shape=(2, 1), mode='ignore_borders')],
             [images], Images2Neibs)
         shape = (100, 40, 5, 3)
-        images = numpy.ones(shape).astype('float32')
+        images = np.ones(shape).astype('float32')
         x = T.ftensor4()
         self._compile_and_check(
             [x], [images2neibs(
@@ -417,19 +539,37 @@ class T_Images2Neibs(unittest_tools.InferShapeTester):
             [images], Images2Neibs)
 
         shape = (100, 40, 6, 7)
-        images = numpy.ones(shape).astype('float32')
+        images = np.ones(shape).astype('float32')
         x = T.ftensor4()
         self._compile_and_check(
             [x], [images2neibs(
                 x, neib_shape=(2, 2), mode='ignore_borders')],
             [images], Images2Neibs)
         shape = (100, 40, 5, 10)
-        images = numpy.ones(shape).astype('float32')
+        images = np.ones(shape).astype('float32')
         x = T.ftensor4()
         self._compile_and_check(
             [x], [images2neibs(
                 x, neib_shape=(3, 3), mode='wrap_centered')],
             [images], Images2Neibs)
+        shape = (100, 40, 6, 4)
+        images = np.ones(shape).astype('float32')
+        x = T.ftensor4()
+        self._compile_and_check(
+            [x], [images2neibs(x, neib_shape=(2, 1), mode='half')],
+            [images], Images2Neibs)
+        self._compile_and_check(
+            [x], [images2neibs(x, neib_shape=(2, 3), mode='half')],
+            [images], Images2Neibs)
+        shape = (100, 40, 6, 5)
+        images = np.ones(shape).astype('float32')
+        x = T.ftensor4()
+        self._compile_and_check(
+            [x], [images2neibs(x, neib_shape=(2, 1), mode='full')],
+            [images], Images2Neibs)
+        self._compile_and_check(
+            [x], [images2neibs(x, neib_shape=(2, 3), mode='full')],
+            [images], Images2Neibs)
 
 if __name__ == '__main__':
     unittest.main()
