@@ -4,7 +4,7 @@
 
 from __future__ import absolute_import, print_function, division
 import unittest
-import numpy
+import numpy as np
 from nose.plugins.skip import SkipTest
 from six.moves import reduce
 
@@ -42,16 +42,16 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         c = tensor.iscalar('c')
         f = theano.function([c, x, y], ifelse(c, x, y), mode=self.mode)
         self.assertFunctionContains1(f, self.get_ifelse(1))
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
 
         xlen = rng.randint(200)
         ylen = rng.randint(200)
 
-        vx = numpy.asarray(rng.uniform(size=(xlen,)), self.dtype)
-        vy = numpy.asarray(rng.uniform(size=(ylen,)), self.dtype)
+        vx = np.asarray(rng.uniform(size=(xlen,)), self.dtype)
+        vy = np.asarray(rng.uniform(size=(ylen,)), self.dtype)
 
-        assert numpy.allclose(vx, f(1, vx, vy))
-        assert numpy.allclose(vy, f(0, vx, vy))
+        assert np.allclose(vx, f(1, vx, vy))
+        assert np.allclose(vy, f(0, vx, vy))
 
     def test_not_lazy_if_inplace(self):
         # Tests that if the outputs are scalars and the graph is big,
@@ -71,16 +71,16 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
                   if isinstance(n.op, IfElse)]
         assert len(ifnode) == 1
         assert not ifnode[0].op.as_view
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
 
         xlen = rng.randint(200)
         ylen = rng.randint(200)
 
-        vx = numpy.asarray(rng.uniform(size=(xlen,)), self.dtype)
-        vy = numpy.asarray(rng.uniform(size=(ylen,)), self.dtype)
+        vx = np.asarray(rng.uniform(size=(xlen,)), self.dtype)
+        vy = np.asarray(rng.uniform(size=(ylen,)), self.dtype)
 
-        assert numpy.allclose(vx, f(1, vx, vy))
-        assert numpy.allclose(vy + sum(range(200)), f(0, vx, vy))
+        assert np.allclose(vx, f(1, vx, vy))
+        assert np.allclose(vy + sum(range(200)), f(0, vx, vy))
 
     def test_mixed_dtype(self):
         x1 = tensor.vector('x1', dtype='int32')
@@ -91,23 +91,23 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         f = theano.function([c, x1, x2, y1, y2],
                             ifelse(c, (x1, x2), (y1, y2)), mode=self.mode)
         self.assertFunctionContains1(f, self.get_ifelse(2))
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
 
         xlen = rng.randint(200)
         ylen = rng.randint(200)
 
-        vx1 = numpy.asarray(rng.uniform(size=(xlen,)) * 3, 'int32')
-        vx2 = numpy.asarray(rng.uniform(size=(xlen,)), self.dtype)
-        vy1 = numpy.asarray(rng.uniform(size=(ylen,)) * 3, 'int32')
-        vy2 = numpy.asarray(rng.uniform(size=(ylen,)), self.dtype)
+        vx1 = np.asarray(rng.uniform(size=(xlen,)) * 3, 'int32')
+        vx2 = np.asarray(rng.uniform(size=(xlen,)), self.dtype)
+        vy1 = np.asarray(rng.uniform(size=(ylen,)) * 3, 'int32')
+        vy2 = np.asarray(rng.uniform(size=(ylen,)), self.dtype)
 
         o1, o2 = f(1, vx1, vx2, vy1, vy2)
-        assert numpy.allclose(vx1, o1)
-        assert numpy.allclose(vx2, o2)
+        assert np.allclose(vx1, o1)
+        assert np.allclose(vx2, o2)
 
         o1, o2 = f(0, vx1, vx2, vy1, vy2)
-        assert numpy.allclose(vy1, o1)
-        assert numpy.allclose(vy2, o2)
+        assert np.allclose(vy1, o1)
+        assert np.allclose(vy2, o2)
 
     def test_lazy_if_on_generics(self):
         x = theano.generic()
@@ -134,24 +134,24 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         # There is only 2 of the 3 ifelse that are moved on the GPU.
         # The one that stay on the CPU is for the shape.
         self.assertFunctionContains(f, self.get_ifelse(1), min=2, max=3)
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
 
         xlen = rng.randint(200)
         ylen = rng.randint(200)
 
-        vx = numpy.asarray(rng.uniform(size=(xlen,)), self.dtype)
-        vy = numpy.asarray(rng.uniform(size=(ylen,)), self.dtype)
+        vx = np.asarray(rng.uniform(size=(xlen,)), self.dtype)
+        vy = np.asarray(rng.uniform(size=(ylen,)), self.dtype)
         gx0, gy0 = f(1, vx, vy)
-        assert numpy.allclose(gx0.shape, vx.shape)
-        assert numpy.allclose(gy0.shape, vy.shape)
-        assert numpy.all(numpy.asarray(gx0) == 1.)
-        assert numpy.all(numpy.asarray(gy0) == 0.)
+        assert np.allclose(gx0.shape, vx.shape)
+        assert np.allclose(gy0.shape, vy.shape)
+        assert np.all(np.asarray(gx0) == 1.)
+        assert np.all(np.asarray(gy0) == 0.)
 
         gx0, gy0 = f(0, vx, vy)
-        assert numpy.allclose(gx0.shape, vx.shape)
-        assert numpy.allclose(gy0.shape, vy.shape)
-        assert numpy.all(numpy.asarray(gx0) == 0.)
-        assert numpy.all(numpy.asarray(gy0) == 1.)
+        assert np.allclose(gx0.shape, vx.shape)
+        assert np.allclose(gy0.shape, vy.shape)
+        assert np.all(np.asarray(gx0) == 0.)
+        assert np.all(np.asarray(gy0) == 1.)
 
     def test_grad_cast_input(self):
         # Tests the gradient when both inputs are on the GPU.
@@ -178,24 +178,24 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
                   if isinstance(x.op, IfElse)][0]
         assert len(ifnode.outputs) == 2
 
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
 
         x1len = rng.randint(200)
         x2len = rng.randint(200)
         y1len = rng.randint(200)
         y2len = rng.randint(200)
 
-        vx1 = numpy.asarray(rng.uniform(size=(x1len,)), self.dtype)
-        vx2 = numpy.asarray(rng.uniform(size=(x2len,)), self.dtype)
-        vy1 = numpy.asarray(rng.uniform(size=(y1len,)), self.dtype)
-        vy2 = numpy.asarray(rng.uniform(size=(y2len,)), self.dtype)
+        vx1 = np.asarray(rng.uniform(size=(x1len,)), self.dtype)
+        vx2 = np.asarray(rng.uniform(size=(x2len,)), self.dtype)
+        vy1 = np.asarray(rng.uniform(size=(y1len,)), self.dtype)
+        vy2 = np.asarray(rng.uniform(size=(y2len,)), self.dtype)
 
         ovx1, ovx2 = f(1, vx1, vx2, vy1, vy2)
         ovy1, ovy2 = f(0, vx1, vx2, vy1, vy2)
-        assert numpy.allclose(vx1, ovx1)
-        assert numpy.allclose(vy1, ovy1)
-        assert numpy.allclose(vx2, ovx2)
-        assert numpy.allclose(vy2, ovy2)
+        assert np.allclose(vx1, ovx1)
+        assert np.allclose(vy1, ovy1)
+        assert np.allclose(vx2, ovx2)
+        assert np.allclose(vy2, ovy2)
 
     def test_multiple_out_grad(self):
         # Tests that we can compute the gradients through lazy if
@@ -209,35 +209,35 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
                             [x1, x2, y1, y2])
 
         f = theano.function([c, x1, x2, y1, y2], grads)
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
 
         lens = [rng.randint(200) for i in range(4)]
-        values = [numpy.asarray(rng.uniform(size=(l,)), theano.config.floatX)
+        values = [np.asarray(rng.uniform(size=(l,)), theano.config.floatX)
                   for l in lens]
         outs_1 = f(1, *values)
         assert all([x.shape[0] == y for x, y in zip(outs_1, lens)])
-        assert numpy.all(outs_1[0] == 1.)
-        assert numpy.all(outs_1[1] == 1.)
-        assert numpy.all(outs_1[2] == 0.)
-        assert numpy.all(outs_1[3] == 0.)
+        assert np.all(outs_1[0] == 1.)
+        assert np.all(outs_1[1] == 1.)
+        assert np.all(outs_1[2] == 0.)
+        assert np.all(outs_1[3] == 0.)
 
         outs_0 = f(0, *values)
         assert all([x.shape[0] == y for x, y in zip(outs_1, lens)])
-        assert numpy.all(outs_0[0] == 0.)
-        assert numpy.all(outs_0[1] == 0.)
-        assert numpy.all(outs_0[2] == 1.)
-        assert numpy.all(outs_0[3] == 1.)
+        assert np.all(outs_0[0] == 0.)
+        assert np.all(outs_0[1] == 0.)
+        assert np.all(outs_0[2] == 1.)
+        assert np.all(outs_0[3] == 1.)
 
     def test_multiple_out_crash(self):
         # This test failed up to commit 2faeb62c38
-        p0 = self.shared(numpy.asarray(numpy.random.random([4, 8]),
-                                       dtype=self.dtype))
-        p1 = self.shared(numpy.asarray(numpy.random.random(8),
-                                       dtype=self.dtype))
-        p2 = self.shared(numpy.asarray(numpy.random.random([8, 3]),
-                                       dtype=self.dtype))
-        p3 = self.shared(numpy.asarray(numpy.random.random(3),
-                                       dtype=self.dtype))
+        p0 = self.shared(np.asarray(np.random.random([4, 8]),
+                                    dtype=self.dtype))
+        p1 = self.shared(np.asarray(np.random.random(8),
+                                    dtype=self.dtype))
+        p2 = self.shared(np.asarray(np.random.random([8, 3]),
+                                    dtype=self.dtype))
+        p3 = self.shared(np.asarray(np.random.random(3),
+                                    dtype=self.dtype))
         p = [p0, p1, p2, p3]
 
         # in my code these vars are the result of applying scan
@@ -264,15 +264,15 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
                             updates=new_updates, mode=self.mode)
         self.assertFunctionContains1(f, self.get_ifelse(4))
 
-        i1 = numpy.asarray(numpy.random.random([19, 4, 8]), dtype=self.dtype)
-        i2 = numpy.asarray(numpy.random.random([19, 8]), dtype=self.dtype)
-        i3 = numpy.asarray(numpy.random.random([19, 8, 3]), dtype=self.dtype)
-        i4 = numpy.asarray(numpy.random.random([19, 3]), dtype=self.dtype)
+        i1 = np.asarray(np.random.random([19, 4, 8]), dtype=self.dtype)
+        i2 = np.asarray(np.random.random([19, 8]), dtype=self.dtype)
+        i3 = np.asarray(np.random.random([19, 8, 3]), dtype=self.dtype)
+        i4 = np.asarray(np.random.random([19, 3]), dtype=self.dtype)
 
         f(i1, i2, i3, i4)
 
     def test_dtype_mismatch(self):
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         data = rng.rand(5).astype(self.dtype)
         x = self.shared(data)
         y = tensor.cast(x * 10, 'int8')
@@ -282,7 +282,7 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         self.assertRaises(TypeError, ifelse, cond, y, x)
 
     def test_ndim_mismatch(self):
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         data = rng.rand(5).astype(self.dtype)
         x = self.shared(data)
         y = tensor.col('y', self.dtype)
@@ -292,7 +292,7 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         self.assertRaises(TypeError, ifelse, cond, y, x)
 
     def test_broadcast_mismatch(self):
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         data = rng.rand(5).astype(self.dtype)
         x = self.shared(data)
         # print x.broadcastable
@@ -307,7 +307,7 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         import theano.sparse
         if not theano.sparse.enable_sparse:
             raise SkipTest("Optimization temporarily disabled")
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         data = rng.rand(2, 3).astype(self.dtype)
         x = self.shared(data)
         y = theano.sparse.matrix('csc', dtype=self.dtype, name='y')
@@ -375,7 +375,7 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         f = theano.function([x1, x2, y1, y2, w1, w2, c], out,
                             allow_input_downcast=True)
         assert isinstance(f.maker.fgraph.toposort()[-1].op, IfElse)
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         vx1 = rng.uniform()
         vx2 = rng.uniform()
         vy1 = rng.uniform()
@@ -383,10 +383,10 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         vw1 = rng.uniform()
         vw2 = rng.uniform()
 
-        assert numpy.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 1),
-                              vx1 * vy1 * vw1)
-        assert numpy.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 0),
-                              vx2 * vy2 * vw2)
+        assert np.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 1),
+                           vx1 * vy1 * vw1)
+        assert np.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 0),
+                           vx2 * vy2 * vw2)
 
     def test_pushout3(self):
         raise SkipTest("Optimization temporarily disabled")
@@ -394,23 +394,23 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         y1 = tensor.scalar('x2')
         y2 = tensor.scalar('y2')
         c = tensor.iscalar('c')
-        two = numpy.asarray(2, dtype=theano.config.floatX)
+        two = np.asarray(2, dtype=theano.config.floatX)
         x, y = ifelse(c, (x1, y1), (two, y2), name='f1')
-        o3 = numpy.asarray(0.3, dtype=theano.config.floatX)
-        o2 = numpy.asarray(0.2, dtype=theano.config.floatX)
+        o3 = np.asarray(0.3, dtype=theano.config.floatX)
+        o2 = np.asarray(0.2, dtype=theano.config.floatX)
         z = ifelse(c, o3, o2, name='f2')
         out = x * z * y
 
         f = theano.function([x1, y1, y2, c], out,
                             allow_input_downcast=True)
         assert isinstance(f.maker.fgraph.toposort()[-1].op, IfElse)
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         vx1 = rng.uniform()
         vy1 = rng.uniform()
         vy2 = rng.uniform()
 
-        assert numpy.allclose(f(vx1, vy1, vy2, 1), vx1 * vy1 * 0.3)
-        assert numpy.allclose(f(vx1, vy1, vy2, 0), 2 * vy2 * 0.2)
+        assert np.allclose(f(vx1, vy1, vy2, 1), vx1 * vy1 * 0.3)
+        assert np.allclose(f(vx1, vy1, vy2, 0), 2 * vy2 * 0.2)
 
     def test_pushout2(self):
         raise SkipTest("Optimization temporarily disabled")
@@ -428,7 +428,7 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         f = theano.function([x1, x2, y1, y2, w1, w2, c], out,
                             allow_input_downcast=True)
         assert isinstance(f.maker.fgraph.toposort()[-1].op, IfElse)
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         vx1 = rng.uniform()
         vx2 = rng.uniform()
         vy1 = rng.uniform()
@@ -439,15 +439,15 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
             vw = vw1
         else:
             vw = vw2
-        assert numpy.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 1),
-                              vx1 * vy1 * vw)
+        assert np.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 1),
+                           vx1 * vy1 * vw)
 
         if vx2 > vy2:
             vw = vw1
         else:
             vw = vw2
-        assert numpy.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 0),
-                              vx2 * vy2 * vw)
+        assert np.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 0),
+                           vx2 * vy2 * vw)
 
     def test_merge_ifs_true_false(self):
         raise SkipTest("Optimization temporarily disabled")
@@ -467,22 +467,21 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
         assert len([x for x in f.maker.fgraph.toposort()
                     if isinstance(x.op, IfElse)]) == 1
 
-        rng = numpy.random.RandomState(utt.fetch_seed())
+        rng = np.random.RandomState(utt.fetch_seed())
         vx1 = rng.uniform()
         vx2 = rng.uniform()
         vy1 = rng.uniform()
         vy2 = rng.uniform()
         vw1 = rng.uniform()
         vw2 = rng.uniform()
-        assert numpy.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 1),
-                              vx1 + vy1 + vw1)
-        assert numpy.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 0),
-                              vx2 + vy2 + vw2)
+        assert np.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 1),
+                           vx1 + vy1 + vw1)
+        assert np.allclose(f(vx1, vx2, vy1, vy2, vw1, vw2, 0),
+                           vx2 + vy2 + vw2)
 
     def test_grad_test_values(self):
-        """
-        Regression test for test values of `ifelse` gradient.
-        """
+        # Regression test for test values of `ifelse` gradient.
+
         backup = theano.config.compute_test_value
         theano.config.compute_test_value = 'raise'
         try:
@@ -494,8 +493,8 @@ class test_ifelse(unittest.TestCase, utt.TestOptimizationMixin):
             theano.config.compute_test_value = backup
 
     def test_grad_int_value(self):
-        w = theano.shared(numpy.random.rand(10))
-        b = theano.shared(numpy.random.rand())
+        w = theano.shared(np.random.rand(10))
+        b = theano.shared(np.random.rand())
         params = [w, b]
 
         x = tensor.vector()
